PlayerMonitor = {};
PlayerMonitor.Init = true;
PlayerMonitor.RemoveOnce = true;

PlayerMonitor_Database = {};

local trace = true;
local debug = true;

--Event methods
function PlayerMonitor.OnLoad()
	PlayerMonitor_SnapshotCount = 1;

	PlayerMonitor.UpdateInterval = .08;
	PlayerMonitor.LastUpdate = 0;

	PlayerMonitor.ShapshotInterval = .15;
	PlayerMonitor.LastSnapshot = 0;
	
	PlayerMonitor.runTime = 0;

	PlayerMonitor.setTimestamp();
	
	PlayerMonitor_Database['error'] = {};

	local status, err = pcall(function() 
	   PlayerMonitor.trace("loaded");
	   RegisterEventHandler(SystemData.Events["SOCIAL_FRIENDS_UPDATED"], "PlayerMonitor.FRIENDLIST_UPDATE" );
	end)
	if(err ~= nil) then
		PlayerMonitor.err(err);
	end
end

function PlayerMonitor.setTimestamp()
	local status, err = pcall(function() 
		local lastEntry = TextLogGetNumEntries("Chat") - 1;
		Calendar.SetTodaysDate();
		if(lastEntry > 0) then
	        local lastTime = TextLogGetEntry("Chat", lastEntry);
	        local hours,mins,secs = lastTime:match(L"([0-9]+):([0-9]+):([0-9]+)");
		    PlayerMonitor.Timestamp = Calendar.todaysMonth .. "-" .. Calendar.todaysDay .. "-" .. Calendar.todaysYear .. " " ..  tostring(hours) .. ":" .. tostring(mins) .. ":" .. tostring(secs); 
	      end
	end)
	if(err ~= nil) then
		PlayerMonitor.err(err);
	end
end

function PlayerMonitor.OnUpdate(elapsed)
	local status, err = pcall(function()
		PlayerMonitor.setTimestamp();
		PlayerMonitor.runTime = (elapsed / 60.0) + PlayerMonitor.runTime;
		if(PlayerMonitor.Init) then
			if(PlayerMonitor.runTime > .5) then
				if(PlayerMonitor.RemoveOnce) then
		   			PlayerMonitor.Init = PlayerMonitor.RemoveFriends();
		   			PlayerMonitor.RemoveOnce = false;
		   		end
		   	end
		else
			if(PlayerMonitor.runTime - PlayerMonitor.LastUpdate > PlayerMonitor.UpdateInterval) then
				PlayerMonitor.LastUpdate = PlayerMonitor.runTime;
				local status, err = pcall(function() 
				   local friendCount = SocialWindowTabFriends.NumberOfFriends;
				   if (friendCount < 40) then
					  PlayerMonitor.AddNewFriend();
				   end   	
				end)
				if(err ~= nil) then
					PlayerMonitor.err(err);
				end
			end
			if(PlayerMonitor.runTime - PlayerMonitor.LastSnapshot > PlayerMonitor.ShapshotInterval) then
				PlayerMonitor.LastSnapshot = PlayerMonitor.runTime;
		    	PlayerMonitor.RecordStatus(PlayerMonitor.Timestamp);
			end
		end
	end)
	if(err ~= nil) then
		PlayerMonitor.err(err);
	end
end

function PlayerMonitor.FRIENDLIST_UPDATE()
   -- figure out how to handle an update: note that this is
   -- fired whenever we log in, open the friends window, 
   -- switch to the tab, add a friend, remove a friend, a friend comes
   -- online, or a friend goes offline
	local status, err = pcall(function() 
	   if(PlayerMonitor.Init) then
          PlayerMonitor.Init = PlayerMonitor.RemoveFriends();
	   end
	end)
	if(err ~= nil) then
		PlayerMonitor.err(err);
	end
end

--Helper methods
function PlayerMonitor.AddNewFriend()
	if(Census_NewPlayers == nil) then 
		PlayerMonitor.err("Census_NewPlayers is nil!");
	end
    local count = #Census_NewPlayers;
    if (count > 0) then    	
	local status, err = pcall(function() 
	       PlayerMonitor.debug(count .. " players in Census_NewPlayers list");
    		 local name = nil;
	    	 local x = 0;
    		 while (name == nil) do
			 x = math.random(count);
			 PlayerMonitor.debug("Picked player " .. x .. " of " .. count);
			 name = Census_NewPlayers[x];
	    	 end
      	 PlayerMonitor.debug("adding " .. tostring(name) .. " to friend list");
         	 SocialWindowTabFriendsPlayerSearchEditBox.Text = name;
 	  	 SocialWindowTabFriends.AddFriend();
 	  	 table.remove(Census_NewPlayers, x);
	end)
	if(err ~= nil) then
		PlayerMonitor.err(err);
		return false;
	end
    else
    	return false;
    end
	return true;
end

function PlayerMonitor.RemoveFriends()
   -- remove everyone from my current friendlist
   local friendCount = SocialWindowTabFriends.NumberOfFriends;
   PlayerMonitor.trace("RemoveFriends called");
   PlayerMonitor.debug("total friend count: " .. friendCount);
   
   if (friendCount > 0) then
		 SocialWindowTabFriends.SelectedPlayerDataIndex = 1;
		 SocialWindowTabFriends.RemoveFriend();
		 return true;
   end

   return false;
end

function PlayerMonitor.RecordPlayer(name, time, area, connected)
	local status, err = pcall(function() 
		local last = tonumber(PlayerMonitor_Database[name]["lastsession"]);
		local lasttime = tonumber(PlayerMonitor_Database[name]["lastmeasure"]);
		if (connected == 1) then
			if PlayerMonitor_Database[name][last] == nil then
		  		PlayerMonitor_Database[name][last] = {};
		  	else
		  		if(lasttime > 1 and PlayerMonitor_Database[name][last][lasttime-1]["area"] == area) then
					-- PlayerMonitor.debug("No change for " .. name .. ", still in area " .. area);
		  			return;
		  		else
		  			PlayerMonitor.debug(name .. " has moved to area " .. area);
					PlayerMonitor.debug(tostring(name) .. " : " .. tostring(time) .. " : " .. tostring(area) .. " : " .. tostring(connected));
		  		end
	  	    end
	
			if PlayerMonitor_Database[name][last][lasttime] == nil then
				PlayerMonitor_Database[name][last][lasttime] = {};
		  	end
	
			PlayerMonitor_Database[name][last][lasttime]["time"] = time;
		  	PlayerMonitor_Database[name][last][lasttime]["area"] = area;
		  	PlayerMonitor_Database[name][last][lasttime]["connected"] = connected;
		  	PlayerMonitor_Database[name]["lastmeasure"] = lasttime + 1;
	  	else
			if PlayerMonitor_Database[name][last] == nil then
		 		PlayerMonitor_Database[name][last] = {};
			end
		 
			if PlayerMonitor_Database[name][last][lasttime - 1] ~= nil then
				-- don't record if we don't have one session time at least
				PlayerMonitor_Database[name][last][lasttime] = {};
		 		PlayerMonitor_Database[name][last][lasttime]["time"] = time;
		 		PlayerMonitor_Database[name][last][lasttime]["connected"] = "offline";
		 		PlayerMonitor_Database[name]["lastsession"] = last + 1;
		 		PlayerMonitor_Database[name]["lastmeasure"] = 1;
			end
	  	end
	end)
	if(err ~= nil) then
		PlayerMonitor.err(err);
	end
end

-- checks the current status of the friends in the friend list
function PlayerMonitor.RecordStatus(time)
	local status, err = pcall(function() 
	   local friendCount = SocialWindowTabFriends.NumberOfFriends;
	
	   if (friendCount > 0) then
		  PlayerMonitor.trace(time .. ": recording status of " .. friendCount .. " friends.");
		  -- walk through the list of friends
	   		local friendsList = GetFriendsList();
			for index, character in ipairs(friendsList) do
		      	local connected = 1;
		      	if(character.zoneID <= 0) then
		      		connected = 0;
		      	end
				-- if they're not connected, remove them and add a new one
				if(connected == 0) then
				   PlayerMonitor.debug("Removing " .. tostring(character.name) .. " (" .. index .. "), no longer connected.");
				   SocialWindowTabFriends.SelectedPlayerDataIndex = index;
				   SocialWindowTabFriends.RemoveFriend();
				end
				
			  if(character.name ~= nil) then
			  	local currentName = tostring(character.name);
			    -- set up to record info on this player
				if PlayerMonitor_Database[currentName] == nil then
				   PlayerMonitor_Database[currentName] = {};
				   PlayerMonitor_Database[currentName]["lastsession"] = 1;
				   PlayerMonitor_Database[currentName]["lastmeasure"] = 1;
				end
			 
				-- we want to record time, zone, connected?
				--PlayerMonitor.debug(tostring(currentName) .. " : " .. tostring(time) .. " : " .. character.zoneID .. " : " .. connected);
				PlayerMonitor.RecordPlayer(currentName, time, character.zoneID, connected);
			 end
		  end
	   end
	end)
	if(err ~= nil) then
		PlayerMonitor.err(err);
	end
end

--Util

function PlayerMonitor.trace(msg)
	if(trace) then
		EA_ChatWindow.Print( L"[PlayerMonitor]: " .. towstring(tostring(msg)), ChatSettings.Channels[SystemData.ChatLogFilters.CHANNEL_7].id);
	end
end

function PlayerMonitor.debug(msg)
	if(debug) then
		EA_ChatWindow.Print( L"[PlayerMonitor]: " .. towstring(tostring(msg)), ChatSettings.Channels[SystemData.ChatLogFilters.CHANNEL_8].id);
	end
end

function PlayerMonitor.err(msg)
	table.insert(PlayerMonitor_Database['error'], msg);
	EA_ChatWindow.Print( L"[PlayerMonitor]: " .. towstring(tostring(msg)), ChatSettings.Channels[SystemData.ChatLogFilters.CHANNEL_9].id);
end
